#!/usr/bin/env python

# Projector Tool Version 0.1
# Designed by Luke November 2009

# Commands are based on http://www.thinkwiki.org/wiki/Xorg_RandR_1.2
# Structure based on the PyGTK examples http://www.pygtk.org/pygtk2tutorial/index.html
# Free software, use a your own risk.

import pygtk
pygtk.require('2.0')
import gtk
import os

def goodResolution(res):
	# checks for good resolution number, such as 800x600:
	a=res[0:res.find("x")]
	b=res[res.find("x")+1:]
	# both must be valid numbers:
	return (a.isdigit() and b.isdigit())

class ProjectorUtil:
	def clone(self, widget, data=None):
		print "Mirroring Screens"
		# possible output channels:
		os.system("xrandr --output LVDS --auto --output VGA --auto --same-as LVDS")
		os.system("xrandr --output LVDS --auto --output VGA-0 --auto --same-as LVDS")
		os.system("xrandr --output LVDS --auto --output DVI-0 --auto --same-as LVDS")
		os.system("xrandr --output LVDS --auto --output TMDS-1 --auto --same-as LVDS")
		
	def disconnect(self, widget, data=None):
		print "Disabling Screen"
		# possible output channels:
		os.system("xrandr --output VGA --off")
		os.system("xrandr --output VGA-0 --off")
		os.system("xrandr --output DVI-0 --off")
		os.system("xrandr --output TMDS-1 --off")
		
	def addRightOf(self, widget, data=None):
		print "Second screen right"
		resolution = self.resText.get_text()
		if goodResolution(resolution):
			os.system("xrandr --output VGA --mode "+resolution+" --right-of LVDS")
			os.system("xrandr --output VGA-0 --mode "+resolution+" --right-of LVDS")
			os.system("xrandr --output DVI-0 --mode "+resolution+" --right-of LVDS")
			os.system("xrandr --output TMDS-1 --mode "+resolution+" --right-of LVDS")
			msg = gtk.MessageDialog(self.window, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO, gtk.BUTTONS_CLOSE, "Drag windows to the right to view on projector.\nYou can also hold down alt and drag any window to the other screen.")
			msg.run()
			msg.destroy()
		else:
			msg = gtk.MessageDialog(self.window, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, "Invalid resolution.")
			msg.run()
			msg.destroy()
		
	def addLeftOf(self, widget, data=None):
		print "Second screen left"
		resolution = self.resText.get_text()
		if goodResolution(resolution):
			os.system("xrandr --output VGA --mode "+resolution+" --left-of LVDS")
			os.system("xrandr --output VGA-0 --mode "+resolution+" --left-of LVDS")
			os.system("xrandr --output DVI-0 --mode "+resolution+" --left-of LVDS")
			os.system("xrandr --output TMDS-1 --mode "+resolution+" --left-of LVDS")
			msg = gtk.MessageDialog(self.window, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO, gtk.BUTTONS_CLOSE, "Drag windows to the left to view on projector.\nYou can also hold down alt and drag any window to the other screen.")
			msg.run()
			msg.destroy()
		else:
			msg = gtk.MessageDialog(self.window, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, "Invalid resolution.")
			msg.run()
			msg.destroy()

	def delete_event(self, widget, event, data=None):
		#Nothing to save, return false:
		return False

	def destroy(self, widget, data=None):
		print "Exiting."
		gtk.main_quit()

	def __init__(self):
		# Main window
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.set_title("Projector Utility")
		
		# window-manager close
		self.window.connect("delete_event", self.delete_event)
		
		# cleanup
		self.window.connect("destroy", self.destroy)
	
		# Window border
		self.window.set_border_width(7)
		
		# Create buttons.
		self.button = gtk.Button("Mirror screens")
		self.button.show()
		self.dButton = gtk.Button("Turn off second-screen output")
		self.dButton.show()
		self.secondLabel = gtk.Label("\nDouble-screen options:")
		self.secondLabel.show()
		
		self.quitButton = gtk.Button("Done")
		self.quitButton.show()
		
		self.mainBox = gtk.VBox(False, 0)
		
		self.resLabel = gtk.Label("Resolution: ")
		self.resLabel.show()
		self.resText = gtk.Entry(max=12)
		self.resText.set_text("800x600")
		self.resText.show()
		self.resBox = gtk.HBox(False, 0)
		self.resBox.pack_start(self.resLabel,False,True,0)
		self.resBox.pack_start(self.resText,True,True,0)
		self.resBox.show()
		
		self.infoLabel = gtk.Label("Plug in the projector, then choose a command:")
		self.infoLabel.show()
		
		self.leftButton = gtk.Button("< 2nd screen to Left")
		self.leftButton.connect("clicked", self.addLeftOf, self.window)
		self.leftButton.show()
		self.rightButton = gtk.Button("2nd Screen to Right >")
		self.rightButton.connect("clicked", self.addRightOf, self.window)
		self.rightButton.show()
		self.rlBox = gtk.HBox(False, 0)
		self.rlBox.pack_start(self.leftButton,True,True,0)
		self.rlBox.pack_start(self.rightButton,True,True,0)
		self.rlBox.show()
		
		self.blank = gtk.Label("   ")
		self.blank.show()
		# Buttons that recieve "clicked" signal:
		self.button.connect("clicked", self.clone, None)
		self.dButton.connect("clicked", self.disconnect, None)
		
		# Quit button
		self.quitButton.connect_object("clicked", gtk.Widget.destroy, self.window)
		
		# Finalize objects:
		
		self.mainBox.pack_start(self.infoLabel, True, True, 0)
		self.mainBox.pack_start(self.button, True, True, 0)
		self.mainBox.pack_start(self.dButton, True, True, 0)
		self.mainBox.pack_start(self.secondLabel, True, True, 0)
		self.mainBox.pack_start(self.rlBox,True,True, 0)
		self.mainBox.pack_start(self.resBox, True, True, 0)
		self.mainBox.pack_start(self.blank, True, True, 0)
		self.mainBox.pack_start(self.quitButton, True, True, 0)
		self.mainBox.show()
		
		self.window.add(self.mainBox)
	
		# and the window
		self.window.show()

	def main(self):
		# Required method waits for event.
		gtk.main()

#Run:
if __name__ == "__main__":
	prog = ProjectorUtil()
	prog.main()
